@@ -3,6 +3,7 @@ import asyncio
 import logging
 from uuid import UUID
 
+from grpc import RpcError
 from tabulate import tabulate
 
 from preseem import NetworkMetadataReference
@@ -12,12 +13,24 @@ from preseem_protobuf.model.element_pb2 import ElementStatus
 # entities from being created.
 sync_lock = None
 
+async def call_and_log_error(co):
+    """Call a GRPC coroutine and log any errors from it."""
+    try:
+        return await co
+    except asyncio.CancelledError:
+        raise
+    except RpcError as err:
+        logging.warning('sync_preseem_model: error calling service: %s (%s)', err.code(), err.details())
+    except Exception as err:
+        logging.warning('sync_preseem_model: %s error calling service: %s', type(err).__name__, err)
+    return None
 
 async def sync_preseem_model(preseem_model,
                              nm_model,
                              company_uuid,
                              system=None,
                              delete=False,
+                             noact=False,
                              verbose=0):
     """
     Update the Preseem model to keep it in sync with metadata references.
@@ -96,9 +109,12 @@ async def sync_preseem_model(preseem_model,
 
         # Make sure we have all sites we need.
         for site_name in new_sites:
-            site = site_names[site_name] = await preseem_model.Site.Create(
-                company_uuid=company_uuid, name=site_name)
-            model_sites[site.uuid] = site
+            logging.info('sync_preseem_model: create site "%s"', site_name)
+            if not noact:
+                site = site_names[site_name] = await call_and_log_error(preseem_model.Site.Create(
+                    company_uuid=company_uuid, name=site_name))
+                if site:
+                    model_sites[site.uuid] = site
 
         # Create or update elements
         ap_model_refs = nm_model.refs.get('ap_model') or {}
@@ -121,22 +137,27 @@ async def sync_preseem_model(preseem_model,
                 if updates:
                     logging.info('sync_preseem_model: update element "%s" %s',
                                  UUID(bytes=elem.uuid), updates)
-                    await preseem_model.Element.Update(
-                        company_uuid=company_uuid, uuid=elem.uuid, **updates)
+                    if not noact:
+                        await call_and_log_error(preseem_model.Element.Update(
+                            company_uuid=company_uuid, uuid=elem.uuid, **updates))
             else:
                 if ip:
                     logging.info(
                         'sync_preseem_model: create element "%s" in site "%s" with management IP %s',
                         name, site_name, ip)
-                    elem = await preseem_model.Element.Create(
-                        company_uuid=company_uuid,
-                        site_uuid=site_uuid,
-                        name=name,
-                        management_ip=ip,
-                        status=ElementStatus.ELEMENT_STATUS_UNKNOWN)
-                    logging.info('sync_preseem_model: created element "%s"',
-                                 UUID(bytes=elem.uuid))
-                    linked_elems.add(elem.uuid)
+                    if not noact:
+                        elem = await call_and_log_error(
+                            preseem_model.Element.Create(
+                                company_uuid=company_uuid,
+                                site_uuid=site_uuid,
+                                name=name,
+                                management_ip=ip,
+                                status=ElementStatus.ELEMENT_STATUS_UNKNOWN)
+                        )
+                        if elem:
+                            logging.info('sync_preseem_model: created element "%s"',
+                                         UUID(bytes=elem.uuid))
+                            linked_elems.add(elem.uuid)
             if elem:
                 model_ref = ap_model_refs.get(ap_ref.value)
                 elem_uuid = str(UUID(bytes=elem.uuid))
@@ -155,14 +176,16 @@ async def sync_preseem_model(preseem_model,
                 logging.info(
                     "sync_preseem_model: remove billing ID from element %s",
                     UUID(bytes=elem.uuid))
-                await preseem_model.Element.Update(company_uuid=company_uuid,
-                                                   uuid=elem.uuid,
-                                                   site_uuid=None,
-                                                   name=None)
+                if not noact:
+                    await call_and_log_error(preseem_model.Element.Update(company_uuid=company_uuid,
+                                                       uuid=elem.uuid,
+                                                       site_uuid=None,
+                                                       name=None))
             else:
                 # Deactivate elements that haven't been polled
                 logging.info("sync_preseem_model: deactivate element %s",
                              UUID(bytes=elem.uuid))
-                await preseem_model.Element.Update(company_uuid=company_uuid,
-                                                   uuid=elem.uuid,
-                                                   inactive=True)
+                if not noact:
+                    await call_and_log_error(preseem_model.Element.Update(company_uuid=company_uuid,
+                                                       uuid=elem.uuid,
+                                                       inactive=True))
